package ap.proof;

import ap.terfor.{Formula, TermOrder, Conjunction}
import ap.util.Debug

object WeakenTree {
  
  private val AC = Debug.AC_PROOF_TREE

  def apply(subtree : ProofTree,
            disjunct : Formula,
            order : TermOrder) : ProofTree = subtree match {
    case subtree : WeakenTree =>
      WeakenTree(subtree.subtree,
                 Conjunction.disj(Array(subtree.disjunct, disjunct),
                                  order),
                 order)
    case _ => new WeakenTree(subtree, disjunct, order)
  }
  
}

/**
 * <code>ProofTreeOneChild</code> that weakens the closing constraint of its
 * <code>subtree</code> by disjunctively adding a formula
 */
class WeakenTree private (val subtree : ProofTree,
                          val disjunct : Formula,
                          protected val order : TermOrder)
      extends ProofTreeOneChild {

  protected val subtreeOrder : TermOrder = order
  
  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(WeakenTree.AC,
                   (order isSortingOf disjunct) &&
                   !subtree.isInstanceOf[WeakenTree])
  //////////////////////////////////////////////////////////////////////////////
  
  lazy val closingConstraint : Formula =
    Conjunction.disj(Array(subtree.closingConstraint, disjunct),
                     subtreeOrder)

  def update(newSubtree : ProofTree) : ProofTree =
    WeakenTree(newSubtree, disjunct, order)
  
  //////////////////////////////////////////////////////////////////////////////

  override def toString : String = {
    "WeakenConstraintTree (" + disjunct + ") (" + closingConstraint +
      ") {\n" + subtree + "\n}"
  }

}
