package ap.terfor;

/**
 * Trait for objects that can be sorted by a <code>TermOrder</code>. It is a
 * design decision that the used order cannot be queried from an object (so that
 * it must not be stored in the object). It can be checked whether an object is
 * consistent with a certain given order, however.
 */
trait Sorted[+T] {
  /**
   * Re-sort an object with a new <code>TermOrder</code>. It is guaranteed that
   * the result <code>isSortedBy(order)</code>
   */
  def sortBy(order : TermOrder) : T
  
  def isSortedBy(order : TermOrder) : Boolean
}
